//------------------------------------------------------------------------------
//
//      
//     Andy BitOff : bitoff@pisem.net
//
//      
//      http://www.delphifr.com/codes/EFFET-EAU-AVEC-GETDIBITS-SETDIBITS_42445.aspx
//    barbichette
//      http://www.delphifr.com/auteur/BARBICHETTE/14124.aspx
//
//------------------------------------------------------------------------------


unit WaterCircle;

interface

uses
  Windows, Graphics;

const
  maxcailloux = 5;

type
  ttab = array [0 .. 0] of Integer;
  ptab = ^ttab;
  TPoscailloux = array [1 .. maxcailloux] of TRect;
  TWaterCircle = class
  private
    FPuissance: Integer;
    BitInfo: TBitmapInfo;
    im: ptab;
    Cailloux: ptab;
    Poscailloux: TPoscailloux;
    cwi, che: Integer;
    cpt: byte;
    wi, he: Integer;
    Step: Integer;
    mx, my, mc: Integer;
    Count: Integer;
    procedure SetPuissance(const Value: Integer);
  public
    constructor Create(Bitmap: TBitmap);
    destructor Destroy; override;
    procedure DrawImage(Bitmap: TBitmap);
    procedure Drop(X, Y: Integer);
    //  
    property Puissance: Integer read FPuissance write SetPuissance;
  end;


implementation

const
  tablecailloux: TPoscailloux =
    ((Left: 000; Top: 0; Right: 058; Bottom: 76), (Left: 059; Top: 0;
      Right: 140; Bottom: 64), (Left: 141; Top: 0; Right: 210; Bottom: 69),
    (Left: 211; Top: 0; Right: 289; Bottom: 76), (Left: 290; Top: 0;
      Right: 340; Bottom: 106));

constructor TWaterCircle.Create(Bitmap: TBitmap);
var
  bit: tbitmap;
begin
  Puissance := 500;
  cpt := 0;
  Count := 0;
  Step := 0;
  mc := 0;
  Poscailloux := tablecailloux;
  bit := tbitmap.create;
  bit.Assign(Bitmap);
  bit.PixelFormat := pf32bit;
  getmem(cailloux, bit.Width * bit.Height * 4);
  cwi := bit.Width;
  che := bit.Height;
  with bitinfo.bmiHeader do begin
    biSize := sizeof(bitinfo.bmiHeader);
    biWidth := bit.Width;
    biHeight := -bit.Height;
    biPlanes := 1;
    biBitCount := 32;
    biCompression := BI_RGB;
    biSizeImage := 0;
    biXPelsPerMeter := 1;
    biYPelsPerMeter := 1;
    biClrUsed := 0;
    biClrImportant := 0;
  end;
  GetDIBits(bit.Canvas.Handle, bit.Handle, 0, bit.Height, cailloux, bitinfo,  DIB_RGB_COLORS);
  wi := Bitmap.Width;
  he := Bitmap.Height;
  getmem(im, wi * he * 4 * 4);
  with bitinfo.bmiHeader do begin
    biWidth := wi;
    biHeight := -he;
    biSizeImage := 0;
  end;
  GetDIBits(Bitmap.Canvas.Handle, Bitmap.Handle, 0, he, im, bitinfo, DIB_RGB_COLORS);
  fillchar(im[wi * he * 2], wi * he * 4 * 2, 0);
end;

destructor TWaterCircle.Destroy;
begin
  freemem(im);
  freemem(Cailloux);
end;

procedure TWaterCircle.DrawImage(Bitmap: TBitmap);
var
  cp, sp: Integer;
  i, j: Integer;
  wn: Integer;
  nw: dword;
  dx, dy: Integer;

  function getpix(X, Y: Integer): Integer;
  var
    tx, ty, k: Integer;
  begin
    result := im[X + Y * wi]; ;
    for k := 1 to maxcailloux do
      if (X >= poscailloux[k].Left) and (Y >= poscailloux[k].Top) and
        (X <= poscailloux[k].Right) and (Y <= poscailloux[k].Bottom) then begin
        tx := X - poscailloux[k].Left + tablecailloux[k].Left;
        ty := Y - poscailloux[k].Top + tablecailloux[k].Top;
        if cailloux[tx + cwi * ty] <> $FF00FF then
          result := cailloux[tx + cwi * ty];
      end;
  end;

begin
  Inc(count);
  cp := wi * he * 2 + cpt * wi * he;
  sp := wi * he * 2 + (1 - cpt) * wi * he;
  for i := 1 to he - 2 do begin
    wn := i * wi;
    for j := 1 to wi - 2 do begin
      Inc(wn);
      nw := ((im[cp + wn - wi - 1] + im[cp + wn - wi] + im[cp + wn - wi + 1]
            + im[cp + wn - 1] + im[cp + wn + 1] + im[cp + wn + wi - 1] + im
            [cp + wn + wi] + im[cp + wn + wi + 1]) shr 2) - im[sp + wn];
      im[sp + wn] := nw;
      if im[sp + wn] < 0 then
        im[sp + wn] := 0;
    end;
  end;
  for i := 1 to he - 2 do begin
    wn := i * wi;
    for j := 1 to wi - 2 do begin
      inc(wn);
      dx := im[sp + wn] - im[sp + wn + wi];
      dy := im[sp + wn] - im[sp + wn + 1];
      dx := dx div 4 + j;
      if dx >= wi then dx := wi - 1;
      if dx < 0 then dx := 0;
      dy := dy div 4 + i;
      if dy >= he then dy := he - 1;
      if dy < 0 then dy := 0;
      im[wi * he + j + i * wi] := getpix(dx, dy);
    end;
  end;
  SetDIBits(Bitmap.Canvas.Handle, Bitmap.Handle, 0, he, @(im[wi * he]), bitinfo, DIB_RGB_COLORS);
  cpt := 1 - cpt;
end;

procedure TWaterCircle.Drop(X, Y: Integer);
begin
  if x < 1 then exit;
  if y < 1 then exit;
  if x > wi - 2 then exit;
  if y > he - 2 then Exit;
  im[wi * he * 2 + wi * Y + X] := puissance;
  im[wi * he * 2 + wi * Y + X + 1] := puissance shr 1;
  im[wi * he * 2 + wi * Y + X - 1] := puissance shr 1;
  im[wi * he * 2 + wi * Y + X + wi] := puissance shr 1;
  im[wi * he * 2 + wi * Y + X - wi] := puissance shr 1;
  if mc = 0 then
    exit;
  mx := X - mx;
  my := Y - my;
  poscailloux[mc].Left := poscailloux[mc].Left + mx;
  poscailloux[mc].Right := poscailloux[mc].Right + mx;
  poscailloux[mc].Top := poscailloux[mc].Top + my;
  poscailloux[mc].Bottom := poscailloux[mc].Bottom + my;
  mx := X;
  my := Y;
end;

procedure TWaterCircle.SetPuissance(const Value: Integer);
begin
  if (Value > 200) and (Value < 5000) then
    FPuissance := Value;
end;

end.
